@@ -38,24 +38,23 @@
 #include "menu-util.h"
 #include "panel-globals.h"
 #include "panel-profile.h"
+#include "panel-lockdown.h"
 
 #define PANEL_MENU_BAR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BAR, PanelMenuBarPrivate))
 
 #define MENU_FLAGS (MAIN_MENU_SYSTEM | MAIN_MENU_DISTRIBUTION_SUB)
 
-enum {
-	PROP_0,
-};
-
 struct _PanelMenuBarPrivate {
 	AppletInfo            *info;
 	PanelWidget           *panel;
 
 	GtkWidget             *applications_menu;
 	GtkWidget             *actions_menu;
+	GtkWidget             *actions_item;
 };
 
 static GObjectClass *parent_class;
+static void panel_menu_bar_append_actions_menu (PanelMenuBar *menubar);
 
 static void
 panel_menu_bar_show_applications_menu (PanelMenuBar *menubar,
@@ -70,117 +69,123 @@ panel_menu_bar_show_applications_menu (PanelMenuBar *menubar,
 	while (GTK_MENU_SHELL (menu)->children)
 		gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
 
-	create_root_menu (menu, NULL, TRUE, MENU_FLAGS, FALSE);
+	create_root_menu (menu, menubar->priv->panel, TRUE, MENU_FLAGS, FALSE);
 }
 
 static void
-panel_menu_bar_append_action_item (PanelMenuBar *menubar,
-				   GtkWidget    *menu,
-				   const char   *title,
-				   const char   *stock_id,
-				   const char   *tooltip,
-				   const char   *drag_id,
-				   GCallback     callback)
+panel_menu_bar_append_action_item (PanelMenuBar          *menubar,
+				   PanelActionButtonType  action_type,
+				   GCallback              callback)
 {
 	GtkWidget *item;
 
 	item = gtk_image_menu_item_new ();
 	setup_stock_menu_item (item,
 			       panel_menu_icon_get_size (),
-			       stock_id,
-			       title,
+			       panel_action_get_stock_icon (action_type),
+			       panel_action_get_text (action_type),
 			       TRUE);
 
-	gtk_tooltips_set_tip (panel_tooltips, item, tooltip, NULL);
+	gtk_tooltips_set_tip (panel_tooltips,
+			      item,
+			      panel_action_get_tooltip (action_type),
+			      NULL);
 
-	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
 	g_signal_connect (item, "activate", callback, NULL);
 	g_signal_connect (G_OBJECT (item), "button_press_event",
 			  G_CALLBACK (menu_dummy_button_press_event), NULL);
-	setup_internal_applet_drag (item, drag_id);
+	setup_internal_applet_drag (item,
+				    panel_action_get_drag_id (action_type));
+}
+
+static void
+panel_menu_bar_recreate_actions_menu (PanelMenuBar *menubar)
+{
+	if (menubar->priv->actions_menu) {
+		gtk_widget_destroy (menubar->priv->actions_menu);
+		menubar->priv->actions_menu = NULL;
+
+		panel_menu_bar_append_actions_menu (menubar);
+	}
 }
 
 static void
 panel_menu_bar_append_actions_menu (PanelMenuBar *menubar)
 {
 	GtkWidget *item;
+	gboolean   enable_log_out;
+	gboolean   enable_lock_screen;
 
-	menubar->priv->actions_menu = panel_create_menu ();
+	if (!menubar->priv->actions_menu) {
+		menubar->priv->actions_menu = panel_create_menu ();
 
-	/* intercept all right button clicks makes sure they don't
-	   go to the object itself */
-	g_signal_connect (G_OBJECT (menubar->priv->actions_menu), "button_press_event",
-			  G_CALLBACK (menu_dummy_button_press_event), NULL);
+		/* intercept all right button clicks makes sure they don't
+	   	go to the object itself */
+		g_signal_connect (G_OBJECT (menubar->priv->actions_menu), "button_press_event",
+			  	G_CALLBACK (menu_dummy_button_press_event), NULL);
 
-	g_signal_connect (menubar->priv->actions_menu, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &menubar->priv->actions_menu);
+		g_signal_connect (menubar->priv->actions_menu, "destroy",
+			  	G_CALLBACK (gtk_widget_destroyed),
+			  	&menubar->priv->actions_menu);
+	}
 
-	if ( ! panel_profile_get_inhibit_command_line ()) {
-		panel_menu_bar_append_action_item (
-						   menubar,
-						   menubar->priv->actions_menu,
-						   _("Run Application..."),
-						   PANEL_STOCK_RUN,
-						   _("Run an Application by entering a command"),
-						   "ACTION:run:NEW",
+	if (!panel_lockdown_get_disable_command_line ()) {
+		panel_menu_bar_append_action_item (menubar,
+						   PANEL_ACTION_RUN,
 						   G_CALLBACK (panel_action_run_program));
 
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
+		gtk_widget_show (item);
 	}
 
 	if (panel_is_program_in_path  ("gnome-search-tool"))
-		panel_menu_bar_append_action_item (
-			menubar,
-			menubar->priv->actions_menu,
-			_("Search for Files..."),
-			PANEL_STOCK_SEARCHTOOL,
-			_("Find files, folders, and documents on your computer"),
-			"ACTION:search:NEW",
-			G_CALLBACK (panel_action_search));
+		panel_menu_bar_append_action_item (menubar,
+						   PANEL_ACTION_SEARCH,
+						   G_CALLBACK (panel_action_search));
 
 	panel_recent_append_documents_menu (menubar->priv->actions_menu);
 
 	if (panel_is_program_in_path ("gnome-panel-screenshot")) {
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
+		gtk_widget_show (item);
 
-		panel_menu_bar_append_action_item (
-			menubar,
-			menubar->priv->actions_menu,
-			_("Take Screenshot..."),
-			PANEL_STOCK_SCREENSHOT,
-			_("Take a screenshot of your desktop"),
-			"ACTION:screenshot:NEW",
-			G_CALLBACK (panel_action_screenshot));
+		panel_menu_bar_append_action_item (menubar,
+						   PANEL_ACTION_SCREENSHOT,
+						   G_CALLBACK (panel_action_screenshot));
 	}
 
-	item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
+	enable_log_out = !panel_lockdown_get_disable_log_out ();
+	enable_lock_screen =
+		(!panel_lockdown_get_disable_lock_screen () &&
+		 panel_is_program_in_path  ("xscreensaver"));
+		
 
-	if (panel_is_program_in_path  ("xscreensaver"))
-		panel_menu_bar_append_action_item (
-			menubar,
-			menubar->priv->actions_menu,
-			_("Lock Screen"), 
-			PANEL_STOCK_LOCKSCREEN, 
-			_("Protect your computer from unauthorized use"),
-			"ACTION:lock:NEW",
-			G_CALLBACK (panel_action_lock_screen));
+	if (enable_log_out || enable_lock_screen) {
+		item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_append (GTK_MENU_SHELL (menubar->priv->actions_menu), item);
+		gtk_widget_show (item);
+	}
 
-	panel_menu_bar_append_action_item (
-			menubar,
-			menubar->priv->actions_menu,
-			_("Log Out"),
-			PANEL_STOCK_LOGOUT,
-			_("Quit from the GNOME desktop"),
-			"ACTION:logout:NEW",
-			G_CALLBACK (panel_action_logout));
+	if (enable_lock_screen)
+		panel_menu_bar_append_action_item (menubar,
+						   PANEL_ACTION_LOCK,
+						   G_CALLBACK (panel_action_lock_screen));
 
-	item = gtk_menu_item_new_with_label (_("Actions"));
-	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menubar->priv->actions_menu);
-	gtk_menu_shell_append (GTK_MENU_SHELL (menubar), item);
+	if (enable_log_out) {
+		panel_menu_bar_append_action_item (menubar,
+						   PANEL_ACTION_LOGOUT,
+						   G_CALLBACK (panel_action_logout));
+	}
+
+	if (!menubar->priv->actions_item) {
+		menubar->priv->actions_item = gtk_menu_item_new_with_label (_("Actions"));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menubar), menubar->priv->actions_item);
+	}
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menubar->priv->actions_item),
+				   menubar->priv->actions_menu);
 }
 
 static void
@@ -220,8 +225,12 @@ panel_menu_bar_instance_init (PanelMenuBar      *menubar,
 				  G_CALLBACK (panel_menu_bar_show_applications_menu), menubar);
 	g_signal_connect_swapped (menubar->priv->applications_menu, "hide",
 				  G_CALLBACK (gtk_menu_shell_deselect), menubar);
-	
+	menubar->priv->actions_menu = NULL;
+	menubar->priv->actions_item = NULL;
 	panel_menu_bar_append_actions_menu (menubar);
+
+	panel_lockdown_notify_add (G_CALLBACK (panel_menu_bar_recreate_actions_menu),
+				   menubar);
 }
 
 static void
@@ -234,10 +243,12 @@ panel_menu_bar_parent_set (GtkWidget *widget,
 
 	menubar->priv->panel = (PanelWidget *) widget->parent;
 
-	g_object_set_data (G_OBJECT (menubar->priv->applications_menu),
-			   "menu_panel", menubar->priv->panel);
-	g_object_set_data (G_OBJECT (menubar->priv->actions_menu),
-			   "menu_panel", menubar->priv->panel);
+	panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->applications_menu),
+				       "menu_panel",
+				       menubar->priv->panel);
+	panel_applet_menu_set_recurse (GTK_MENU (menubar->priv->actions_menu),
+				       "menu_panel",
+				       menubar->priv->panel);
 }
 
 static void
@@ -308,8 +319,11 @@ panel_menu_bar_load (PanelWidget *panel,
 		return;
 	}
 
-	panel_applet_add_callback (
-		menubar->priv->info, "help", GTK_STOCK_HELP, _("_Help"));
+	panel_applet_add_callback (menubar->priv->info,
+				   "help",
+				   GTK_STOCK_HELP,
+				   _("_Help"),
+				   NULL);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (menubar), FALSE, TRUE);
 }
@@ -331,9 +345,8 @@ panel_menu_bar_create (PanelToplevel *toplevel,
 	char *id;
 
 	id = panel_profile_prepare_object (PANEL_OBJECT_MENU_BAR, toplevel, position, FALSE);
-
-	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
+	g_free (id);
 }
 
 void
@@ -350,7 +363,7 @@ panel_menu_bar_invoke_menu (PanelMenuBar *menubar,
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (menubar));
 
-	panel_show_help (screen, "wgospanel.xml", "gospanel-37");
+	panel_show_help (screen, "user-guide.xml", "gospanel-37");
 }
 
 void
