@@ -29,9 +29,11 @@
 
 G_BEGIN_DECLS
 
-void panel_run_dialog_present           (GdkScreen  *screen);
+void panel_run_dialog_present           (GdkScreen  *screen,
+					 guint32    activate_time);
 void panel_run_dialog_present_with_text (GdkScreen  *screen,
-					 const char *text);
+					 const char *text,
+					 guint32    activate_time);
 
 G_END_DECLS
 
